@@ -17,6 +17,8 @@
 
 #    Copyright 2010 Jussi Holm
 
+import os
+
 import gtk
 import hildon
 
@@ -87,6 +89,12 @@ class MainWindow(hildon.StackableWindow):
 
         self.menu.append(self.deleteButton)
 
+        self.aboutButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.aboutButton.set_label('About')
+        self.aboutButton.connect("clicked", lambda x: AboutDialog(self))
+
+        self.menu.append(self.aboutButton)
+
         self.menu.show_all()
         self.set_app_menu(self.menu)
 
@@ -197,10 +205,11 @@ class Graph(hildon.PannableArea):
         self.graphWidth = 800
         self.graphHeight = 424
         self.set_property('mov-mode', hildon.MOVEMENT_MODE_HORIZ)
+        self.set_property('vscrollbar-policy', gtk.POLICY_NEVER)
         self.set_size_request(self.graphWidth, self.graphHeight)
         self.area = gtk.DrawingArea()
         self.add_with_viewport(self.area)
-        
+
         self.area.connect("expose-event", self.redrawGraph)
 
         self.get_hadjustment().connect("value-changed", self.moveLabels)
@@ -210,6 +219,9 @@ class Graph(hildon.PannableArea):
         self.gridColor = self.area.get_colormap().alloc_color(40*256, 40*256, 40*256)
         self.labelsColor = self.area.get_colormap().alloc_color(120*256, 120*256, 120*256)
 
+        self.chargingColor = self.area.get_colormap().alloc_color(25*256, 35*256, 0)
+        self.fullColor = self.area.get_colormap().alloc_color(10*256, 40*256, 0)
+
         self.dataset = 'hal.battery.charge_level.percentage'
         self.visibleHours = 25
 
@@ -269,21 +281,26 @@ class Graph(hildon.PannableArea):
         else:
             self.xGridInterval = 6
             self.xLabelInterval = 24
-            
-
+           
         areaWidth = max((self.xMax - self.xMin) / 3600 * (self.graphWidth/self.visibleHours), self.graphWidth - 10)
 
         self.drawAreaInfo = DrawAreaInfo((areaWidth, self.graphHeight),
                                          (areaWidth, self.graphHeight-40),
                                          (0, 14),
-                                         (self.graphWidth-10, self.graphHeight-10))
+                                         (self.graphWidth, self.graphHeight-10))
 
         self.pixmap = gtk.gdk.Pixmap(None, areaWidth, self.graphHeight, 16)
         self.area.set_size_request(areaWidth, self.graphHeight)
 
         self.datas = []
         for segment in rawData:
-            self.datas.append(self.rescaleData(segment, self.drawAreaInfo, self.xMin, self.xMax, yMin, yMax))
+            self.datas.append(DataRendererValue(segment, self.drawAreaInfo,
+                                                (self.xMin, self.xMax),
+                                                (yMin, yMax)))
+        self.chargeInfo = []
+        for segment in self.getChargeInfo(now):
+            self.chargeInfo.append(DataRendererStatus(segment, self.drawAreaInfo, (self.xMin, self.xMax)))
+            
         self.yGrid = self.yGridSegments(self.drawAreaInfo, yMin, yMax, yLabelsInterval)
         self.xGrid = self.xGridSegments(self.drawAreaInfo, self.xMin, self.xMax, self.xGridInterval)
         self.yLabels = self.createYLabels(yLabels)
@@ -297,15 +314,15 @@ class Graph(hildon.PannableArea):
                                    self.drawAreaInfo.canvasSize[0],
                                    self.drawAreaInfo.canvasSize[1])
 
+        for d in self.chargeInfo:
+            d.render(self.pixmap, {(True, True): self.fullColor, (True, False): self.chargingColor})
+        
         gc.foreground = self.gridColor
         self.pixmap.draw_segments(gc, self.yGrid)
         self.pixmap.draw_segments(gc, self.xGrid)
 
-        gc.foreground = self.dataColor
         for d in self.datas:
-            if len(d):
-                # gtk outputs assertion error if lines is an empty array
-                self.pixmap.draw_lines(gc, d)
+            d.render(self.pixmap, self.dataColor)
 
         gc.foreground = self.labelsColor
         self.drawYLabels(gc, self.yLabels, self.yGrid)
@@ -389,14 +406,69 @@ class Graph(hildon.PannableArea):
             ret.append((areaInfo.offset[0], y, areaInfo.drawAreaSize[0]-1, y))
         return ret
 
-#    def chargeInfo(self, now):
-#        rawData = dataStorage.getObservationsWithBreaks(('hal.battery.rechargeable.is_charging',
-#                                                         'hal.battery.rechargeable.is_discharging'),
-#                                                         now - 30*86400, now)
-#        charges = []
-#        fulls = []
-#        for segment in 
-        
+    def getChargeInfo(self, now):
+        rawData = dataStorage.getObservations(('hal.battery.rechargeable.is_charging',
+                                               'hal.battery.rechargeable.is_discharging',
+                                               'internal.graph.break'),
+                                               now - 30*86400, now)
+        if len(rawData) == 0:
+            return []
+        segment = []
+        segments = [segment]
+        for point in rawData:
+            segment.append(point)
+            if point[2] == 'internal.graph.break':
+                segment = []
+                segments.append(segment)
+
+        # Some serious list processing voodoo follows.
+
+        statuses = [] #[((charging, discharging), start, end), (...), ...]
+        for segment in [x for x in segments if len(x) > 0]:
+            intervalStart = segment[0][0]
+            if segment[-1][2] == 'internal.graph.break':
+                intervalEnd = dataStorage.getPreviousObservationTime(segment[-1][0])
+            else:
+                intervalEnd = dataStorage.getPreviousObservationTime(now)
+            if intervalEnd == None or intervalStart >= intervalEnd:
+                continue
+
+            statusList = []
+            lastStatus = ((False, True), intervalStart, intervalStart)
+            for point in segment:
+                if point[2] == 'hal.battery.rechargeable.is_charging':
+                    if point[1] != lastStatus[0][0]:
+                        if point[0] > lastStatus[2]:
+                            statusList.append((lastStatus[0], lastStatus[1], point[0]))
+                        lastStatus = ((bool(point[1]), lastStatus[0][1]), point[0], point[0])
+                    else:
+                        lastStatus = (lastStatus[0], lastStatus[1], point[0])
+                elif point[2] == 'hal.battery.rechargeable.is_discharging':
+                    if point[1] != lastStatus[0][1]:
+                        if point[0] > lastStatus[2]:
+                            statusList.append((lastStatus[0], lastStatus[1], point[0]))
+                        lastStatus = ((lastStatus[0][0], bool(point[1])), point[0], point[0])
+                    else:
+                        lastStatus = (lastStatus[0], lastStatus[1], point[0])
+            statusList.append((lastStatus[0], lastStatus[1], intervalEnd))
+            statuses.append([x for x in statusList if x[1] < x[2]])
+        return [x for x in statuses if len(x) > 0]
+
+#        for segment in statuses:
+#            print ""
+#            for status in segment:
+#                if status[0] == (True, False):
+#                    descr = "Charging"
+#                elif status[0] == (True, True):
+#                    descr = "Full"
+#                elif status[0] == (False, True):
+#                    descr = "Discharging"
+#                else:
+#                    descr = "W00t? No battery?"
+#                print "%s (%d minutes) %s - %s" % (descr, (status[2] - status[1]) / 60,
+#                                                   datetime.datetime.fromtimestamp(status[1], beye.utc).astimezone(beye.local),
+#                                                   datetime.datetime.fromtimestamp(status[2], beye.utc).astimezone(beye.local))
+       
     def createXLabels(self, xMin, xMax, modulo):
         hours = self.getHoursInInterval(xMin, xMax, 1)
         pContext = self.area.create_pango_context()
@@ -481,6 +553,114 @@ class Graph(hildon.PannableArea):
         deltaForward = datetime.timedelta(0, 0, 0, 0, 0, 1)
         return dt + deltaForward - deltaBack
 
+class AboutDialog(gtk.AboutDialog):
+    def __init__(self, parent):
+        gtk.Dialog.__init__(self)
+        self.set_name("battery-eye")
+        self.set_logo_icon_name("battery-eye")
+
+        self.set_copyright(beye.__copyright__)
+        self.set_version(beye.__version__)
+        self.set_website(beye.__url__)
+
+        stat = os.stat(beye.dbFilePath)
+        self.set_comments('Database size: %dkB, %d records' % (stat[6] / 1024, dataStorage.countObservations()))
+
+        #gtk.about_dialog_set_url_hook(lambda dialog, link, user_data: webbrowser.open(link), None)
+        self.set_license('''The battery-eye software is distributed under GPL version 3. For more information, see
+<http://www.gnu.org/licenses/>
+
+The battery-eye icon is a derivative based on original work by
+rutty <http://www.flickr.com/photos/rutty/438995413>, and is distributed under a Creative Commons license:
+<http://creativecommons.org/licenses/by-nc-sa/2.0/>
+
+battery-eye is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+''')
+        self.set_wrap_license(True)
+        
+        self.show_all()
+        self.run()
+        self.destroy()
+
+class DataRendererValue(object):
+    def __init__(self, rawData, areaInfo, timeBounds, valueBounds):
+        self.rawData = rawData
+        self.areaInfo = areaInfo
+        self.timeBounds = timeBounds
+        self.valueBounds = valueBounds
+        self.scaledData = self.rescaleData(self.rawData, self.areaInfo, self.timeBounds, self.valueBounds)
+
+    def rescaleData(self, points, areaInfo, xBounds, yBounds):
+        if len(points) == 0:
+            return []
+        xDistance = xBounds[1] - xBounds[0]
+        yDistance = yBounds[1] - yBounds[0]
+        if xDistance <= 0 or yDistance <= 0:
+            # Let's not divide by zero
+            return []
+
+        scaleX = float(areaInfo.drawAreaSize[0]-1) / xDistance
+        scaleY = float(areaInfo.drawAreaSize[1]-1) / yDistance
+
+        offX = self.areaInfo.offset[0]
+        offY = self.areaInfo.offset[1]
+
+        def rescalePoint(point):
+            x = scaleX * float(point[0] - xBounds[0])
+            y = areaInfo.drawAreaSize[1] - scaleY * float(point[1] - yBounds[0])
+            return (int(x) + offX, int(y) + offY)
+
+        return map(rescalePoint, points)
+
+    def render(self, drawable, color):
+        gc = drawable.new_gc(foreground=color, line_width=1, join_style=gtk.gdk.JOIN_ROUND)
+        color2 = drawable.get_colormap().alloc_color(int(color.red*0.5),
+                                                     int(color.green*0.5),
+                                                     int(color.blue*0.5))
+        gc2 = drawable.new_gc(foreground=color2, line_width=2, join_style=gtk.gdk.JOIN_ROUND)
+
+        if len(self.scaledData) == 1:
+            drawable.draw_points(gc2, self.scaledData)
+            drawable.draw_points(gc, self.scaledData)
+        elif len(self.scaledData) > 1:
+            drawable.draw_lines(gc2, self.scaledData)
+            drawable.draw_lines(gc, self.scaledData)
+
+class DataRendererStatus(object):
+    def __init__(self, rawData, areaInfo, timeBounds):
+        self.rawData = rawData
+        self.areaInfo = areaInfo
+        self.timeBounds = timeBounds
+        self.scaledData = self.rescaleData(self.rawData, self.areaInfo, self.timeBounds)
+
+    def rescaleData(self, statuses, areaInfo, xBounds):
+        if len(statuses) == 0:
+            return []
+        xDistance = xBounds[1] - xBounds[0]
+        if xDistance <= 0:
+            # Let's not divide by zero
+            return []
+
+        scaleX = float(areaInfo.drawAreaSize[0]-1) / xDistance
+
+        offX = self.areaInfo.offset[0]
+
+        def rescaleStatus(status):
+            x1 = scaleX * float(status[1] - xBounds[0])
+            x2 = scaleX * float(status[2] - xBounds[0])
+            return (status[0], int(x1), int(x2))
+
+        return map(rescaleStatus, statuses)
+
+    def render(self, drawable, colorMap):
+        for status in self.scaledData:
+            if (not colorMap.has_key(status[0])) or colorMap[status[0]] == None:
+                continue
+            gc = drawable.new_gc(foreground=colorMap[status[0]])
+            drawable.draw_rectangle(gc, True,
+                                    status[1], self.areaInfo.offset[1],
+                                    status[2] - status[1], self.areaInfo.offset[1] + self.areaInfo.drawAreaSize[1] - 14)
+
 if __name__ == "__main__":
     dataStorage = beye.DataStorage(beye.setupAndGetDbPath(), [beye.DataSourceHal()])
     mainwin = MainWindow()
