@@ -72,7 +72,13 @@ class DataStorage(object):
         self.__getObservations(cursor, type, start, end)
         ret = []
         for row in cursor.fetchall():
-            ret.append((int(row[0]), int(row[1]), row[2]))
+            try:
+                ret.append((int(row[0]), int(row[1]), row[2]))
+            except TypeError:
+                if row[1] == None:
+                    ret.append((int(row[0]), None, row[2]))
+                else:
+                    raise TypeError("Query returned unknown data")
         return ret
 
     def getObservationsWithBreaks(self, type, start, end):
@@ -95,6 +101,19 @@ class DataStorage(object):
                 segment.append((int(row[0]), int(row[1]), row[2]))
         return ret
 
+    def getPreviousObservationTime(self, timestamp):
+        cursor = self.con.cursor()
+        cursor.execute('select max(timestamp) from observation where timestamp < ?', (timestamp, ))
+        try:
+            return cursor.fetchone()[0]
+        except:
+            return None
+
+    def countObservations(self):
+        cursor = self.con.cursor()
+        cursor.execute('select count(id) from observation')
+        return cursor.fetchone()[0]
+
     def deleteObservations(self, olderThan):
         cursor = self.con.cursor()
         cursor.execute('delete from observation where timestamp < ?',
