@@ -23,11 +23,19 @@ from timezone import local, utc
 import os
 import os.path
 
+__author__    = 'Jussi Holm <jussi@pokko.net>'
+__version__   = '0.6.5'
+__date__      = '2010-03-21'
+__copyright__ = 'Copyright 2010 Jussi Holm'
+__licence__   = 'GNU General Public License, version 3 or later'
+__url__       = 'https://battery-eye.garage.maemo.org/'
+
+dbFilePath = os.path.join(os.path.expanduser('~/.config/battery-eye'), 'beyed.sqlite')
+
 def setupAndGetDbPath():
-    dbDir = os.path.expanduser('~/.config/battery-eye')
+    dbDir = os.path.dirname(dbFilePath)
     if not os.path.isdir(dbDir):
         os.makedirs(dbDir)
     
-    dbFile = os.path.join(dbDir, 'beyed.sqlite')
-    return dbFile
+    return dbFilePath
 
