@@ -32,7 +32,7 @@ import beye
 
 if __name__ == "__main__":
     dbFile = beye.setupAndGetDbPath()
-    ds = beye.DataSourceHal()
-    storage = beye.DataStorage(dbFile, [ds])
-    daemon = beye.Daemon(storage, [ds])
+    ds = [beye.DataSourceInternal(), beye.DataSourceHal()]
+    storage = beye.DataStorage(dbFile, ds)
+    daemon = beye.Daemon(storage, ds)
     daemon.run()
